@@ -4,6 +4,11 @@ from astroquery.simbad import Simbad
 
 
 class SimbadHarvester(AbstractHarvester):
+    """
+    The ``SimbadHarvester`` is the interface to the SIMBAD catalog. At present, it is only queryable by identifier. For
+    information regarding identifier format, please see http://simbad.u-strasbg.fr/simbad/sim-fid or
+    https://astroquery.readthedocs.io/en/latest/simbad/simbad.html.
+    """
     name = 'Simbad'
 
     def __init__(self, *args, **kwargs):
@@ -26,5 +31,5 @@ class SimbadHarvester(AbstractHarvester):
         target.pm_ra = result.get('PMRA')
         target.pm_dec = result.get('PMDEC')
         target.distance = result.get('Distance_distance')
-        target.identifier = result.get('ID', b'').decode('UTF-8').split(',')[0].replace(' ', '')
+        target.name = result.get('ID', b'').decode('UTF-8').split(',')[0].replace(' ', '')
         return target
