@@ -32,6 +32,7 @@ urlpatterns = [
     path('catalogs/', include('tom_catalogs.urls')),
     path('observations/', include('tom_observations.urls', namespace='observations')),
     path('dataproducts/', include('tom_dataproducts.urls', namespace='dataproducts')),
+    path('publications/', include('tom_publications.urls', namespace='publications')),
     path('users/', UserListView.as_view(), name='user-list'),
     path('users/<int:pk>/changepassword/', UserPasswordChangeView.as_view(), name='admin-user-change-password'),
     path('users/create/', UserCreateView.as_view(), name='user-create'),
@@ -42,7 +43,7 @@ urlpatterns = [
     path('groups/<int:pk>/delete/', GroupDeleteView.as_view(), name='group-delete'),
     path('accounts/login/', LoginView.as_view(), name='login'),
     path('accounts/logout/', LogoutView.as_view(), name='logout'),
-    path('comment/<pk>/delete', CommentDeleteView.as_view(), name='comment-delete'),
+    path('comment/<int:pk>/delete', CommentDeleteView.as_view(), name='comment-delete'),
     path('admin/', admin.site.urls),
     # The static helper below only works in development see
     # https://docs.djangoproject.com/en/2.1/howto/static-files/#serving-files-uploaded-by-a-user-during-development
