@@ -1,8 +1,10 @@
 from django import forms
+from django.contrib.auth.models import Group
+from django.conf import settings
 
-from .models import DataProductGroup, DataProduct
-from tom_targets.models import Target
+from tom_dataproducts.models import DataProductGroup, DataProduct
 from tom_observations.models import ObservationRecord
+from tom_targets.models import Target
 
 
 class AddProductToGroupForm(forms.Form):
@@ -29,11 +31,18 @@ class DataProductUploadForm(forms.Form):
             attrs={'multiple': True}
         )
     )
-    tag = forms.ChoiceField(
-        choices=DataProduct.DATA_PRODUCT_TYPES,
+    data_product_type = forms.ChoiceField(
+        choices=[v for k, v in settings.DATA_PRODUCT_TYPES.items()],
         widget=forms.RadioSelect(),
         required=True
     )
     referrer = forms.CharField(
         widget=forms.HiddenInput()
     )
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        if not settings.TARGET_PERMISSIONS_ONLY:
+            self.fields['groups'] = forms.ModelMultipleChoiceField(Group.objects.none(),
+                                                                   required=False,
+                                                                   widget=forms.CheckboxSelectMultiple)
